function [anz_param,name,daten,all_input] = get_data(dateiname)

%Version 1.5
% anz_param ... speichert die Anzahl der bergabeparameter
% name ... Name des Unterprogramms
% daten ... Cell array das alle bergabeparameter gespeichert hat
% all_input ... speichert den gesamten Input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ffnen des Master C - files und Suche nach dem Marker; beziet sich auf
% die Vektoren
%  // begin_marker(In/Out/IO)

master = fopen(dateiname,'r');
tline = 'null';
fehler = 1;
while(isstr(tline))
   tline = fgetl(master);
   if(numel(tline)>15 && strcmp(tline(1:15),'// begin_marker'))
       reihe_vek = tline;
       
       klammer1 = find(reihe_vek == '(');
       klammer2 = find(reihe_vek == ')');
       if (numel(klammer1) ~= 1 || numel(klammer2) ~= 1)
           error('Ungltiger Marker!');
       else
           reihe_vek = reihe_vek(klammer1+1:klammer2-1);
       end
       
       tline = '';
       while(numel(tline) == 0)
           tline = fgetl(master);
           if(numel(tline)>4 && strcmp(tline(1:4),'void'))
               all_input = tline;
               fehler = 0;
           else
               tline(tline == ' ') = [];
           end
       end
   end
end
fclose(master);
if(fehler == 1)
    error('Keinen Korrekten Marker gefunden!');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% anz_param(alle, skalar in, skalar out, vektor in, vektor out, vektor in_out, string in, string in_out, mat in, mat out, mat in_out)
 
anz_param = zeros(12,1);                                                    
% Suche Anzahl der einzelnen bergabeparameter
% Alle
anz = sum(all_input == ',');                                                % Zhlen wie viele ',' zwischen den Klammern sind
if(anz >= 1)
	anz_param(1) = anz + 1;
else
	klammer1 = find(all_input == '(');
	klammer2 = find(all_input == ')');
    if(( klammer2 - klammer1) == 1)                                         % Wenn kein ',' vorhanden knnen es 1 oder 0 Parameter sein
        anz = 0;
        warning('Keine bergabeparameter gefunden!')
    else
        anz = 1;
    end
	anz_param(1) = anz;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Suche Namen des Unterprogramms
if(isnumeric(findstr(all_input,'void')))
	ind1 = findstr(all_input,'void') + 5;
	ind2 = find(all_input == '(') -1;
	name = all_input(ind1:ind2);    
else
    error('Ungltiger Header!');
end
if isempty(name)
    error('Ungltiger Header!');
end
    
if (sum(all_input == '(') + sum(all_input == ')')) ~= 2
    error('Ungltiger Header!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Filter Parameterliste
daten = cell(anz_param(1),6);

anz_array = 1;                                                                % Anzahl der zu bergebenden 1D Arrays (double und strings)

if anz_param(1) > 0
    ind1 = find(all_input == '(')+1;
    ind2 = find(all_input == ')')-1;
    param_list = all_input(ind1:ind2);                                                          
    
    for k = 1 : anz_param(1)
        [str1,param_list] = strtok(param_list,',');                         % str1 kter bergabeparameter (datentype + variablenname)
        [param_typ,param_name] = strtok(str1);                              % Aufteilung von str1 in Datentyp und Name
        
        % Errormeldungen bei falscher Eingabe
        if (~strcmp(param_typ,'double') && ~strcmp(param_typ,'int') && ~strcmp(param_typ,'char')) || isempty(param_typ)
            error('Ungltiger Datentype (Parameter %d)!',k);
        end
        if isempty(param_name)
            error('Ungltiger Name (Parameter %d)!',k);
        end
        param_name(param_name == ' ') = [];
        if isempty(param_name)
            error('Ungltiger Name (Parameter %d)!',k);
        end
        
        daten(k,1) = {num2str(k)};                                          % Nr des bergabeparameters
        daten(k,4) = {param_typ};                                           % Datentype des bergabeparameters
        daten(k,5) = {param_name};                                          % Nane des bergabeparameters  
        
        % Suche nach [] im Parameternamen
        if ( any(param_name == '[') & any(param_name == ']') )
            switch param_typ
                case 'double'
                    daten(k,3) = {'Vektor'};
                    daten(k,2) = {num2str(anz_array)};
                    anz_array = anz_array + 1;
                case 'int' 
                    daten(k,3) = {'Vektor'};
                    daten(k,2) = {num2str(anz_array)};
                    anz_array = anz_array + 1;
                case 'char'
                    daten(k,3) = {'String'};
                    daten(k,2) = {num2str(anz_array)};
                    anz_array = anz_array + 1;
                otherwise
                    error('Datentype wird fr Vektoren nicht unterstzt!');
            end
        % Suche nach * im Parameternamen
        elseif any(param_name == '*')
            if ( sum(param_name == '*') == 1 )
                switch param_typ
                    case 'double'
                        daten(k,3) = {'Skalar'};
                        daten(k,2) = {'Out'};
                        anz_param(3) = anz_param(3) + 1; 
                 case 'int'
                        daten(k,3) = {'Skalar'};
                        daten(k,2) = {'Out'};
                        anz_param(3) = anz_param(3) + 1;
                    case 'char'
                        daten(k,3) = {'String'};
                        daten(k,2) = {num2str(anz_array)};
                        anz_array = anz_array + 1;
                end
            end
            if ( sum(param_name == '*') == 2 ) && findstr(param_name,'**')
                switch param_typ
                    case 'double'
                        daten(k,3) = {'Matrix'};
                        daten(k,2) = {num2str(anz_array)};
                        anz_array = anz_array + 1; 
                    case 'int'
                        daten(k,3) = {'Matrix'};
                        daten(k,2) = {num2str(anz_array)};
                        anz_array = anz_array + 1;
                end
            end    
        else
            % es kann nur mehr Skalar In brig sein
            daten(k,3) = {'Skalar'};
            daten(k,2) = {'In'};
            anz_param(2) = anz_param(2) + 1;
        end
    end
end
anz_array = anz_array - 1;
                 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Filter reihe_vek um daten zu vervollstndigen
if anz_array > 0
    if numel(reihe_vek) < 4
        error('Fehler in Marker!');
    end
    if sum(reihe_vek == ',') + 1 ~= anz_array
        error('Falsche Anzahl an Vektoren im Marker!');
    end
    
    for k = 1 : anz_array
        for l = 1: anz_param(1)
            if(strcmp(cell2mat(daten(l,2)),num2str(k)))
                [str1,reihe_vek] = strtok(reihe_vek,',');                   % Unterschiedliche Parameter trennen                         
                [In_Out,param_name] = strtok(str1);                         % In/Out und Parameternamen trennen
                
                % Leerzeichen streichen
                In_Out(In_Out == ' ') = [];
                param_name(param_name == ' ') = [];
                
                if ~strcmp(cell2mat(daten(l,5)),param_name)
                    error('Keine bereinstimmung von Parametername %d von Marker und Header!',l);
                end   
                
                daten(l,2) = {In_Out};
% anz_param(alle, skalar in, skalar out, vektor in, vektor out, vektor in_out, string in, string out, string in_out)
                
                switch In_Out
                    case 'In'
                        switch cell2mat(daten(l,4))
                            case 'char'
                                anz_param(7) = anz_param(7) + 1;
                            case 'int'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(4) = anz_param(4) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(10) = anz_param(10) + 1;
                                end
                            case 'double'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(4) = anz_param(4) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(10) = anz_param(10) + 1;
                                end  
                            otherwise
                                error('Falscher Datentype!');
                        end
                                 
                    case 'Out'
                        switch cell2mat(daten(l,4))
                            case 'char'
                                anz_param(8) = anz_param(8) + 1;
                            case 'int'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(5) = anz_param(5) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(11) = anz_param(11) + 1;
                                end
                            case 'double'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(5) = anz_param(5) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(11) = anz_param(11) + 1;
                                end
                            otherwise
                                error('Falscher Datentype!');
                        end
                    case 'IO'
                        switch cell2mat(daten(l,4))
                            case 'char'
                                anz_param(9) = anz_param(9) + 1;
                            case 'int'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(6) = anz_param(6) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(12) = anz_param(12) + 1;
                                end
                            case 'double'
                                if strcmp(cell2mat(daten(l,3)),'Vektor')
                                    anz_param(6) = anz_param(6) + 1;
                                end
                                if strcmp(cell2mat(daten(l,3)),'Matrix')
                                    anz_param(12) = anz_param(12) + 1;
                                end
                            otherwise
                                error('Falscher Datentype!');
                        end
                    otherwise
                        error('Fehler in Marker!'); 
                end
            end
        end
    end
end              

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Eingabe durch den Benutzer berprfen lassen
fprintf('__________________________________________________________________________________________\n');
fprintf('\nKontrolle der bergabeparameter\n');
fprintf('\nHeader: %s',all_input);
fprintf('\n%d bergabeparameter: \n',anz_param(1));
for k = 1 : anz_param(1)
    fprintf('\nParameter %s \t %s \t %s \t %s \t %s',cell2mat(daten(k,1)),cell2mat(daten(k,3)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
end
fprintf('\n\n');
reply = '0';

while ( ~strcmp(reply(1),'y') && ~strcmp(reply(1),'n'))
    fprintf('Eingabe korrekt? y/n [y]: ');
    reply = input('', 's');
    if isempty(reply)
        reply = 'y';
    end
    
end
if strcmp(reply(1),'n')
    error('Abbruch durch den Benutzer!');
end