function write_file(anz_param,name,daten,all_input,dateiname);

% Version 1.5
% anz_param ... Anzahl der bergabeparameter
% name ... Name der Funktion
% daten ... Cell array das alle bergabeparameter gespeichert hat
% all_input ... speichert den gesamten Input

% Das Gateway - File heit c2m_<name_des_unterprogramms>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filename = ['c2m_',name,'.c'];                                              % Erstellen des Filenames fr das Gatewayfile
gate = fopen(filename,'wt');

% Header des C - Files
fprintf(gate,'// Gatewayfile fr die Datei %s\n',dateiname);
fprintf(gate,'\n#include "mex.h"');
fprintf(gate,'\n#include "%s"\n',dateiname);
fprintf(gate,'\n%s;\n',all_input);
fprintf(gate,'\nvoid mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )\n{');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variablen Deklarationen
fprintf(gate,'\n// Variablen Deklarationen');                               % bentigt fr nlhs,plhs,nrhs,prhs ; in,out,out vek  
fprintf(gate,'\nint row, col, num_ele, k, l;');                             % Bentigt fr Vektoren
for k = 1 : anz_param(1)                                                    % Schleife ber alle bergabeparameter
    switch cell2mat(daten(k,2))                                             % Unterscheidung um welchen bergabeparameter es sich handelt
        case 'In'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    dec_in_skalar(gate,k,daten);
                case 'Vektor'
                    dec_in_vektor(gate,k,daten);
                case 'String'
                    dec_in_string(gate,k,daten);
                case 'Matrix'
                    dec_in_matrix(gate,k,daten);
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end                    
        case 'Out'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    dec_out_skalar(gate,k,daten);
                case 'Vektor'
                    dec_out_vektor(gate,k,daten);
                case 'String'
                    dec_out_string(gate,k,daten);
                case 'Matrix'
                    dec_out_matrix(gate,k,daten);
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end 
        case 'IO'
            switch cell2mat(daten(k,3))
                case 'Vektor'
                    dec_io_vektor(gate,k,daten);
                case 'String'
                    dec_io_string(gate,k,daten);
                case 'Matrix'
                    dec_io_matrix(gate,k,daten);
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end        
        otherwise
            fclose(gate);
            error('Fehler im Header!')
    end    
end
fprintf(gate,'\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Kontolle der Anzahl der Input und Output Parameter
fprintf(gate,'\n// Check number of arguments');
% Input Parameter
anz_in = anz_param(2)+ anz_param(4)+anz_param(5)+anz_param(6)+anz_param(7)+anz_param(9)+anz_param(10)+anz_param(11)+anz_param(12);
fprintf(gate,'\nif (nrhs != %s)\n{\n\tmexErrMsgTxt("%s input argument(s) required!");\n}',num2str(anz_in),num2str(anz_in));
% Output Parameter
anz_out = anz_param(3)+anz_param(5)+anz_param(6)+anz_param(8)+anz_param(9)+anz_param(11)+anz_param(12);
fprintf(gate,'\nif (nlhs > %s)\n{\n\tmexErrMsgTxt("Too many output arguments!");\n}\n',num2str(anz_out));

% Nur die Anzahl die ins C - file get
% anz_param(alle, 2skalar in, 3skalar out, 4vektor in, 5vektor out, 6vektor in_out, 7string in, 8string out, 9string in_out, 10mat in, 11mat out, 12mat in_out)
in = anz_param(2)+ anz_param(4)+anz_param(6)+anz_param(7)+anz_param(9)+anz_param(10)+anz_param(12);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gateway Code
num = [0,0,0];                                                              % bentigt fr nlhs,plhs,nrhs,prhs ; in,out,out vek
% zahlt mit wie oft ein outvektor verwendet wurde fr den dimensions input        
param = [];                                                                 % In diesem String werden die Namen der bergabeparamter im Gatewayfile gespeichert

for k = 1 : anz_param(1)                                                    % Schleife ber alle bergabeparameter
    switch cell2mat(daten(k,2))                                             % Unterscheidung um welchen bergabeparameter es sich handelt
        case 'In'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    [param,num] = in_skalar(gate,k,daten,anz_param,num,param);
                case 'Vektor'
                    [param,num] = in_vektor(gate,k,daten,anz_param,num,param);
                case 'String'
                    [param,num] = in_string(gate,k,daten,anz_param,num,param);    
                case 'Matrix'
                    [param,num] = in_matrix(gate,k,daten,anz_param,num,param);
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end                    
        case 'Out'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    [param,num] = out_skalar(gate,k,daten,anz_param,num,param);
                case 'Vektor'
                    [param,num] = out_vektor(gate,k,daten,anz_param,num,param,in);
                case 'String'
                    [param,num] = out_string(gate,k,daten,anz_param,num,param);       
                case 'Matrix'
                    [param,num] = out_matrix(gate,k,daten,anz_param,num,param,in);
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end     
        case 'IO'
            switch cell2mat(daten(k,3))
                case 'Vektor'
                    [param,num] = io_vektor(gate,k,daten,anz_param,num,param);
                case 'String'
                    [param,num] = io_string(gate,k,daten,anz_param,num,param);    
                case 'Matrix'
                    [param,num] = io_matrix(gate,k,daten,anz_param,num,param);    
                otherwise
                    fclose(gate);
                    error('Fehler im Header!')
            end 
        otherwise
            fclose(gate);
            error('Fehler im Header!')
    end    
end


% Funktionsaufruf
if ~isempty(param)
    param(end) = [];
end
fprintf(gate,'\n// Programmaufruf');
fprintf(gate,'\n%s(%s);',name,param);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Umspeichern und Speicherplatz freigeben
fprintf(gate,'\n\n// Umspeichern und freigeben des Speichers');
num = zeros(3,1); % input,output,vek in bzw. mat in
for k = 1 : anz_param(1)
    num = copy_und_release(gate,k,daten,anz_param,num,in);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(gate,'\n\n}');
fclose(gate);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Unterprogramme Deklarationen

function dec_in_skalar(gate,k,daten)
% Skalare Input Werte
% Variablenname: x<Parameternummer>
var = ['x',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble %s;',var);   
    case 'int'
        fprintf(gate,'\nint %s;',var);
    otherwise
        disp('Error');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_out_skalar(gate,k,daten)
% Skalare Output Werte
% Variablenname: y<Parameternummer>
var = ['y',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble *%s;',var); 
    case 'int'
        fprintf(gate,'\nint *%s;',var);  
    otherwise
        disp('Error');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_in_vektor(gate,k,daten)
% Vetor Input Werte
% Variablenname: X<Parameternummer>
var = ['X',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble  *%s;',var);   
    case 'int'
        fprintf(gate,'\nint  *%s;',var);
    otherwise
        disp('Error!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_in_string(gate,k,daten)
% String Input Werte
% Variablenname: str_in<Parameternummer>
var = ['str',cell2mat(daten(k,1)),'_In'];                                         % Variablenname

% Write file
fprintf(gate,'\nchar  *%s;',var);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_out_vektor(gate,k,daten)
% Vektor Output Werte
% Variablenname: Y<Parameternummer>
var = ['Y',cell2mat(daten(k,1))];                                           % variablenname
var_numel = ['numel_',cell2mat(daten(k,1))];

%Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble *%s;',var);
        fprintf(gate,'\nint %s;',var_numel);
        
    case 'int'
        fprintf(gate,'\nint *%s;',var);
        fprintf(gate,'\nint %s;',var_numel);
        
    otherwise
        disp('Error!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_out_string(gate,k,daten)
% String Input Werte
% Variablenname: str_out<Parameternummer>
var = ['str',cell2mat(daten(k,1)),'_Out'];                                         % Variablenname

% Write file
fprintf(gate,'\nchar  *%s;',var);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_io_vektor(gate,k,daten)
% Vektor In_Out Werte
% Variablenname: Z<Parameternummer>_In , Z<Parameternummer>_Out
var1 = ['Z',cell2mat(daten(k,1)),'_In'];                                    % Variablenname
var2 = ['Z',cell2mat(daten(k,1)),'_Out'];

% Write file

%Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble  *%s, *%s;',var1,var2);   
    case 'int'
        fprintf(gate,'\nint  *%s, *%s;',var1,var2);
    otherwise
        disp('Error!');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_io_string(gate,k,daten)
% String In Out Werte
% Variablenname: str_in<Parameternummer>, str_out<Parameternummer>
var = ['str',cell2mat(daten(k,1)),'_IO'];                                         % Variablenname
% Write file
fprintf(gate,'\nchar  *%s;',var);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_in_matrix(gate,k,daten)
% Matrix In Werte
% Variablenname: A<Parameternummer>, dummy<Parameternummer>
var1 = ['A',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_',cell2mat(daten(k,1))];
% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble  **%s;',var1);
        fprintf(gate,'\ndouble  *%s;',var2);
    case 'int'
        fprintf(gate,'\nint  **%s;',var1);
        fprintf(gate,'\nint  *%s;',var2);
    otherwise
        disp('Error!');
end     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_out_matrix(gate,k,daten)
% Matrix Out Werte
% Variablenname: B<Parameternummer>, dummy<Parameternummer>,
% size_<Parametername>
var1 = ['B',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_',cell2mat(daten(k,1))];
var3 = ['size_',cell2mat(daten(k,1))];
% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble  **%s;',var1);
        fprintf(gate,'\ndouble  *%s, *%s;',var2,var3);
    case 'int'
        fprintf(gate,'\nint  **%s;',var1);
        fprintf(gate,'\nint  *%s;',var2);
        fprintf(gate,'\ndouble *%s;',var3);
    otherwise
        disp('Error!');
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dec_io_matrix(gate,k,daten);
% Matrix IO Werte
% Variablenname: C<Parameternummer>, dummy_In_<Parameternummer>, dummy_Out_<Parameternummer>,

var1 = ['C',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_In_',cell2mat(daten(k,1))];
var3 = ['dummy_Out_',cell2mat(daten(k,1))];

% write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ndouble  **%s;',var1);
        fprintf(gate,'\ndouble  *%s, *%s;',var2,var3);
    case 'int'
        fprintf(gate,'\nint  **%s;',var1);
        fprintf(gate,'\nint  *%s, *%s;',var2,var3);
    otherwise
        disp('Error!');
end    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Unterprogramme Code

function [param,num] = in_skalar(gate,k,daten,anz_param,num,param)
% Skalare Input Werte
% Variablenname: x<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['x',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\nif ( mxGetNumberOfElements(prhs[%s]) != 1 || mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex scalar double!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = mxGetScalar(prhs[%s]);\n',var,num2str(num(1)));       
    case 'int'
        fprintf(gate,'\nif ( mxGetNumberOfElements(prhs[%s]) != 1 || mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex scalar!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = (int)mxGetScalar(prhs[%s]);\n',var,num2str(num(1)));
    otherwise
        error('Falscher Datentyp!');
end
param = [param,var,','];
num(1) = num(1) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = out_skalar(gate,k,daten,anz_param,num,param)
% Skalare Output Werte
% Variablenname: y<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['y',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\nplhs[%s] = mxCreateDoubleMatrix(1, 1, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = mxGetPr(plhs[%s]);\n',var,num2str(num(2)));    
    case 'int'
        fprintf(gate,'\nplhs[%s] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS ,mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = (int*)mxGetPr(plhs[%s]);\n',var,num2str(num(2)));   
    otherwise
        error('Falscher Datentyp!');
end
param = [param,var,','];
num(2) = num(2) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = in_vektor(gate,k,daten,anz_param,num,param)
% Vetor Input Werte
% Variablenname: X<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['X',cell2mat(daten(k,1))];                                           % Variablenname

% Write file
% Nur geringe Unterschiede zwischen double und int32
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nnum_ele = col*row;');
        fprintf(gate,'\nif ( !(col == 1 && row == num_ele) && !(row == 1 && col == num_ele) )');
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector double!");\n}',num2str(num(1)+1));

        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector double!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = mxGetPr(prhs[%s]);\n',var,num2str(num(1)));
     
    case 'int'
        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nnum_ele = col*row;');
        fprintf(gate,'\nif ( !(col == 1 && row == num_ele) && !(row == 1 && col == num_ele) )');
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector int32!");\n}',num2str(num(1)+1));

        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsInt32(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector int32!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = (int*)mxGetPr(prhs[%s]);\n',var,num2str(num(1)));
        
    otherwise
        disp('Error!');
end

param = [param,var,','];
num(1) = num(1) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = out_vektor(gate,k,daten,anz_param,num,param,in)
% Vektor Output Werte
% Variablenname: Y<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['Y',cell2mat(daten(k,1))];                                           % variablenname
var_numel = ['numel_',cell2mat(daten(k,1))];

%Write file
switch cell2mat(daten(k,4))
    case 'double'
    fprintf(gate,'\nif ( mxGetNumberOfElements(prhs[%s]) != 1 || mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(in + num(3)),num2str(in+num(3)),num2str(in+num(3)));
    fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex scalar!");\n}',num2str(in+num(3)+1));
    fprintf(gate,'\n%s = (int)mxGetScalar(prhs[%s]);',var_numel,num2str(in+num(3)));   % Summe input parameter + num(3)
    
    fprintf(gate,'\nplhs[%s] = mxCreateDoubleMatrix(%s, 1, mxREAL);',num2str(num(2)),var_numel);
    fprintf(gate,'\n%s = mxGetPr(plhs[%s]);\n',var,num2str(num(2)));
         
    case 'int'
    fprintf(gate,'\nif ( mxGetNumberOfElements(prhs[%s]) != 1 || mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(in+num(3)),num2str(in+num(3)),num2str(in+num(3)));
    fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex scalar!");\n}',num2str(in+num(3)+1));
    fprintf(gate,'\n%s = (int)mxGetScalar(prhs[%s]);',var_numel,num2str(in+num(3)));   % Summe input parameter + num(3)
    
    fprintf(gate,'\nplhs[%s] = mxCreateNumericMatrix(%s, 1,mxINT32_CLASS, mxREAL);',num2str(num(2)),var_numel);
    fprintf(gate,'\n%s = (int*)mxGetPr(plhs[%s]);\n',var,num2str(num(2)));
        
    otherwise
        disp('Error!');
end

param = [param,var,','];
num(2) = num(2) + 1;
num(3) = num(3) + 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = io_vektor(gate,k,daten,anz_param,num,param)
% Vektor In_Out Werte
% Variablenname: Z<Parameternummer>_In , Z<Parameternummer>_Out
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var1 = ['Z',cell2mat(daten(k,1)),'_In'];                                    % Variablenname
var2 = ['Z',cell2mat(daten(k,1)),'_Out'];

% Write file
switch cell2mat(daten(k,4))
    case 'double'
        % Fehler Abfrage und Deklaration fr In
        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nnum_ele = col*row;');
        fprintf(gate,'\nif ( !(col == 1 && row == num_ele) && !(row == 1 && col == num_ele) )');
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector double!");\n}',num2str(num(1)+1));

        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector double!");\n}',num2str(num(1)+1));

        fprintf(gate,'\n%s = mxGetPr(prhs[%s]);\n',var1,num2str(num(1)));

        % Deklaration fr Out
        fprintf(gate,'\nplhs[%s] = mxCreateDoubleMatrix(row, col, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = mxGetPr(plhs[%s]);\n',var2,num2str(num(2)));
          
    case 'int'
        % Fehler Abfrage und Deklaration fr In
        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nnum_ele = col*row;');
        fprintf(gate,'\nif ( !(col == 1 && row == num_ele) && !(row == 1 && col == num_ele) )');
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector int32!");\n}',num2str(num(1)+1));

        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsInt32(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 1D vector int32!");\n}',num2str(num(1)+1));

        fprintf(gate,'\n%s = (int*)mxGetPr(prhs[%s]);\n',var1,num2str(num(1)));

        % Deklaration fr Out
        fprintf(gate,'\nplhs[%s] = mxCreateNumericMatrix(row, col, mxINT32_CLASS, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = (int*)mxGetPr(plhs[%s]);\n',var2,num2str(num(2)));
        
    otherwise
        disp('Error!');
end


param = [param,var1,','];
num(1) = num(1) + 1;
num(2) = num(2) + 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = in_string(gate,k,daten,anz_param,num,param)
% string Input Werte
% num = [in,out,out vek]; 
% Variablenname: str_in<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['str',cell2mat(daten(k,1)),'_In'];                                           % Variablenname

% Write file
% Fehlerabfragen
fprintf(gate,'\nif ( mxIsChar(prhs[%s]) != 1 )',num2str(num(1)));
fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a string!");\n}',num2str(num(1)+1));
fprintf(gate,'\nif ( mxGetM(prhs[%s])!=1 )',num2str(num(1)));
fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a row vector!");\n}',num2str(num(1)+1));

fprintf(gate,'\n\n%s = mxArrayToString(prhs[%s]);\n',var,num2str(num(1)));

fprintf(gate,'\nif ( %s == NULL )',var);
fprintf(gate,'\n{\n\tmexErrMsgTxt("Could not convert input argument %s to string!");\n}\n',num2str(num(1)+1));

param = [param,var,','];
num(1) = num(1) + 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = out_string(gate,k,daten,anz_param,num,param)
% String Output Werte
% num = [in,out,out vek]; 
% Variablenname: str_out<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['str',cell2mat(daten(k,1)),'_Out'];                                           % Variablenname

% Write file
fprintf(gate,'\n%s = (char*)mxCalloc(500, sizeof(char));\n',var);
    
param = [param,var,','];
num(2) = num(2) + 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = io_string(gate,k,daten,anz_param,num,param)
% String In Out Werte
% num = [in,out,out vek]; 
% Variablenname: str_in<Parameternummer>,str_out<Parameternummer>
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
var = ['str',cell2mat(daten(k,1)),'_IO'];                                         % Variablenname

% Write File
% Fehlerabfragen; ganz gleich wie bei String In
fprintf(gate,'\nif ( mxIsChar(prhs[%s]) != 1 )',num2str(num(1)));
fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a string!");\n}',num2str(num(1)+1));
fprintf(gate,'\nif ( mxGetM(prhs[%s])!=1)',num2str(num(1)));
fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a row vector!");\n}',num2str(num(1)+1));

fprintf(gate,'\n\n%s = mxArrayToString(prhs[%s]);\n',var,num2str(num(1)));

fprintf(gate,'\nif ( %s == NULL )',var);
fprintf(gate,'\n{\n\tmexErrMsgTxt("Could not convert input argument %s to string!");\n}\n',num2str(num(1)+1));

param = [param,var,','];
num(1) = num(1) + 1;
num(2) = num(2) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = in_matrix(gate,k,daten,anz_param,num,param);
% Matrix In Werte
% num = [in,out,out vek]; 
% Variablenname: A<Parameternummer>, dummy_<Parameternummer>
var1 = ['A',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_',cell2mat(daten(k,1))];

% write file
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 2D matrix double!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = mxGetPr(prhs[%s]);\n',var2,num2str(num(1)));

        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);\n',num2str(num(1)));
        fprintf(gate,'\n%s = (double**) mxMalloc(row*sizeof(double*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(double*) mxMalloc(col*sizeof(double));\n',var1);

        fprintf(gate,'\nfor(l = 0; l < col; l++)');
        fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t\t%s[k][l] = *(%s+k+l*row);\n',var1,var2);
        
    case 'int'
        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsInt32(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 2D matrix int32!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = (int*)mxGetPr(prhs[%s]);\n',var2,num2str(num(1)));

        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);\n',num2str(num(1)));
        fprintf(gate,'\n%s = (int**) mxMalloc(row*sizeof(int*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(int*) mxMalloc(col*sizeof(int));\n',var1);

        fprintf(gate,'\nfor(l = 0; l < col; l++)');
        fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t\t%s[k][l] = *(%s+k+l*row);\n',var1,var2);
    otherwise
        disp('Error!');
end



param = [param,var1,','];
num(1) = num(1) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = out_matrix(gate,k,daten,anz_param,num,param,in)
% Matrix Out Werte
% Variablenname: A<Parameternummer>, dummy<Parameternummer>,
% size_<Parametername>
var1 = ['B',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_',cell2mat(daten(k,1))];
var3 = ['size_',cell2mat(daten(k,1))];

% Write file
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
fprintf(gate,'\nif ( mxGetNumberOfElements(prhs[%s]) != 2 || mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(in+num(3)),num2str(in+num(3)),num2str(in+num(3)));
fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a vector double with two elements [row col]!");\n}',num2str(in+num(3)+1));
fprintf(gate,'\n%s = mxGetPr(prhs[%s]);',var3,num2str(in+num(3)));
fprintf(gate,'\nrow = (int)%s[0];',var3);
fprintf(gate,'\ncol = (int)%s[1];\n',var3);

switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\n%s = (double**) mxMalloc(row*sizeof(double*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(double*) mxMalloc(col*sizeof(double));\n',var1);
        
        fprintf(gate,'\nplhs[%s] = mxCreateDoubleMatrix(row, col, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = mxGetPr(plhs[%s]);\n',var2,num2str(num(2)));
        
    case 'int'
        fprintf(gate,'\n%s = (int**) mxMalloc(row*sizeof(int*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(int*) mxMalloc(col*sizeof(int));\n',var1);
        
        fprintf(gate,'\nplhs[%s] = mxCreateNumericMatrix(row, col, mxINT32_CLASS, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = (int*)mxGetPr(plhs[%s]);\n',var2,num2str(num(2)));
    otherwise
        disp('Error!');
end 
param = [param,var1,','];
num(2) = num(2) + 1;
num(3) = num(3) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [param,num] = io_matrix(gate,k,daten,anz_param,num,param)
% Matrix IO Werte
% Variablenname: C<Parameternummer>, dummy_In_<Parameternummer>, dummy_Out_<Parameternummer>,

var1 = ['C',cell2mat(daten(k,1))];                                         % Variablenname
var2 = ['dummy_In_',cell2mat(daten(k,1))];
var3 = ['dummy_Out_',cell2mat(daten(k,1))];

% write file
fprintf(gate,'\n// Parameter %s \t %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,2)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));
switch cell2mat(daten(k,4))
    case 'double'
        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsDouble(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 2D matrix double!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = mxGetPr(prhs[%s]);\n',var2,num2str(num(1)));

        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);\n',num2str(num(1)));
        fprintf(gate,'\n%s = (double**) mxMalloc(row*sizeof(double*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(double*) mxMalloc(col*sizeof(double));\n',var1);

        fprintf(gate,'\nfor(l = 0; l < col; l++)');
        fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t\t%s[k][l] = *(%s+k+l*row);\n',var1,var2);
        
        fprintf(gate,'\nplhs[%s] = mxCreateDoubleMatrix(row, col, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = mxGetPr(plhs[%s]);\n',var3,num2str(num(2)));
        
    case 'int'
        fprintf(gate,'\nif ( mxIsComplex(prhs[%s]) || !mxIsInt32(prhs[%s]) )',num2str(num(1)),num2str(num(1)));
        fprintf(gate,'\n{\n\tmexErrMsgTxt("Input argument %s must be a noncomplex 2D matrix int32!");\n}',num2str(num(1)+1));
        fprintf(gate,'\n%s = (int*)mxGetPr(prhs[%s]);\n',var2,num2str(num(1)));

        fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
        fprintf(gate,'\nrow = mxGetM(prhs[%s]);\n',num2str(num(1)));
        fprintf(gate,'\n%s = (int**) mxMalloc(row*sizeof(int*));',var1);
        fprintf(gate,'\nfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t%s[k]=(int*) mxMalloc(col*sizeof(int));\n',var1);

        fprintf(gate,'\nfor(l = 0; l < col; l++)');
        fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
        fprintf(gate,'\n\t\t%s[k][l] = *(%s+k+l*row);\n',var1,var2);
        
        fprintf(gate,'\nplhs[%s] = mxCreateNumericMatrix(row, col, mxINT32_CLASS, mxREAL);',num2str(num(2)));
        fprintf(gate,'\n%s = (int*)mxGetPr(plhs[%s]);\n',var3,num2str(num(2)));
 
    otherwise
        disp('Error!');
end

param = [param,var1,','];
num(1) = num(1) + 1;
num(2) = num(2) + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Umspeichern und Speicherplatz freigeben

function num = copy_und_release(gate,k,daten,anz_param,num,in)
                                                          
% fprintf(gate,'\n// Parameter %s \t %s %s',cell2mat(daten(k,1)),cell2mat(daten(k,4)),cell2mat(daten(k,5)));

switch cell2mat(daten(k,2))                                             
        case 'In'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    num(1) = num(1) + 1;
                case 'Vektor'
                    var = ['X',num2str(k)];
                    % fprintf(gate,'\nmxDestroyArray(%s);',var); 
                    num(1) = num(1) + 1;
                    
                case 'String'
                    var = ['str',num2str(k),'_In'];                  
                    fprintf(gate,'\nmxFree(%s);\n',var); 
                    num(1) = num(1) + 1;
                    
                case 'Matrix'
                    var1 = ['A',cell2mat(daten(k,1))]; 
                    fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
                    fprintf(gate,'\nfor(k = 0; k < row; k++)');
                    fprintf(gate,'\n\tmxFree(%s[k]);\n',var1);
                    fprintf(gate,'\nmxFree(%s);\n',var1);
                    num(1) = num(1) + 1;
                    
                otherwise
                    num(1) = num(1) + 1;
            end                    
        case 'Out'
            switch cell2mat(daten(k,3))
                case 'Skalar'
                    var = ['y',num2str(k)];
                    % fprintf(gate,'\nmxDestroyArray(%s);',var); 
                    num(2) = num(2) + 1;
                    
                case 'Vektor'
                    var = ['Y',num2str(k)];
                    % fprintf(gate,'\nmxDestroyArray(%s);',var); 
                    num(2) = num(2) + 1;
                    num(3) = num(3) + 1;
                    
                case 'String'
                    var = ['str',num2str(k),'_Out'];
                    fprintf(gate,'\nplhs[%s] = mxCreateString(%s);',num2str(num(2)),var);
                    fprintf(gate,'\nmxFree(%s);\n',var);
                    num(2) = num(2) + 1;
                    
                case 'Matrix'
                    var1 = ['B',cell2mat(daten(k,1))];                                         % Variablenname
                    var2 = ['dummy_',cell2mat(daten(k,1))];
                    var3 = ['size_',cell2mat(daten(k,1))];
                    fprintf(gate,'\n%s = mxGetPr(prhs[%s]);',var3,num2str(in+num(3)));
                    fprintf(gate,'\nrow = (int)%s[0];',var3);
                    fprintf(gate,'\ncol = (int)%s[1];\n',var3);
                    
                    fprintf(gate,'\nfor(l = 0; l < col; l++)');
                    fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
                    fprintf(gate,'\n\t\t*(%s+k+l*row) = %s[k][l];\n',var2,var1);
                    
                    fprintf(gate,'\nfor(k = 0; k < row; k++)');
                    fprintf(gate,'\n\tmxFree(%s[k]);',var1);
                    fprintf(gate,'\nmxFree(%s);\n',var1);
                    
                    
                otherwise
                    num(2) = num(2) + 1;
                    num(3) = num(3) + 1;
            end 
        case 'IO'
            switch cell2mat(daten(k,3))
                case 'Vektor'
                    var1 = ['Z',num2str(k),'_In'];                                % Variablenname
                    var2 = ['Z',num2str(k),'_Out'];
                    fprintf(gate,'\nfor(k = 0; k < mxGetNumberOfElements(prhs[%s]); k++)\n',num2str(num(1)));
                    fprintf(gate,'\t%s[k] = %s[k];\n',var2,var1);
                    % fprintf(gate,'\nmxDestroyArray(%s);',var1); 
                    % fprintf(gate,'\nmxDestroyArray(%s);',var2); 
                    num(1) = num(1) + 1;
                    num(2) = num(2) + 1;
                                        
                case 'String'
                    var = ['str',num2str(k),'_IO'];
                    fprintf(gate,'\nplhs[%s] = mxCreateString(%s);',num2str(num(2)),var);
                    fprintf(gate,'\nmxFree(%s);',var); 
                    num(1) = num(1) + 1;
                    num(2) = num(2) + 1;
                    
                case 'Matrix'
                    var1 = ['C',cell2mat(daten(k,1))];                                         % Variablenname
                    var3 = ['dummy_Out_',cell2mat(daten(k,1))];
                    
                    fprintf(gate,'\ncol = mxGetN(prhs[%s]);',num2str(num(1)));
                    fprintf(gate,'\nrow = mxGetM(prhs[%s]);',num2str(num(1)));
                    fprintf(gate,'\nfor(l = 0; l < col; l++)');
                    fprintf(gate,'\n\tfor(k = 0; k < row; k++)');
                    fprintf(gate,'\n\t\t*(%s+k+l*row) = %s[k][l];\n',var3,var1);

                    fprintf(gate,'\nfor(k = 0; k < row; k++)');
                    fprintf(gate,'\n\tmxFree(%s[k]);',var1);
                    fprintf(gate,'\nmxFree(%s);',var1);
                    
                    num(1) = num(1) + 1;
                    num(2) = num(2) + 1;
                    
                otherwise
                    num(1) = num(1) + 1;
                    num(2) = num(2) + 1;
            end        
end    